@@ -115,6 +115,51 @@ def load_from_json(filename):
     return json.loads(data)
 
 
+def writeobj_CH(points, hull_simplices, filename = 'this.obj'):
+    '''Writes the triangulated image, which makes a 3-D mesh model, as an .obj file.
+    *for Convex hull function/code'''
+
+    with open(filename, 'w') as f:
+        for i in points:
+            f.write("v  {:.4f} {:.4f} {:.4f}\n".format(*i))
+
+
+        block = """
+vt 1.00 0.00 0.00 
+vt 1.00 1.00 0.00
+vt 0.00 1.00 0.00
+vt 0.00 0.00 0.00
+
+vn 0.00 0.00 -1.00
+vn 0.00 0.00 1.00
+vn 0.00 -1.00 0.00
+vn 1.00 0.00 0.00
+vn 0.00 1.00 0.00
+vn -1.00 0.00 0.00
+
+\n"""
+
+        f.write("\n"+block)
+
+        f.write("\ng Polyhedral\n\n")
+
+        for j in hull_simplices:
+            # the vertex texture (vt) triangle indices which color a specific simplex are [ currently ] being defined at random 
+            rand_t0 = np.random.randint(4)
+            rand_n = np.random.randint(1,7)
+
+
+            j+=1    # hull simplices start at index 1 not 0 (this makes the correction)
+            j1,j2,j3 = j
+
+            facestr = [j1,(rand_t0+0)%4+1,rand_n,\
+                    j2,(rand_t0+1)%4+1,rand_n,\
+                    j3,(rand_t0+2)%4+1,rand_n  
+                    ]
+
+            f.write("f {}/{}/{} {}/{}/{} {}/{}/{}\n".format(*facestr))
+
+
 def writeobj_MC(filename = 'this.obj', *args):
     '''Writes the triangulated image, which makes a 3-D mesh model, as an .obj file.
     *for scikit-learn`s Marching Cubes (MC)'''
@@ -155,23 +200,68 @@ vt 0.00 0.00 0.00
             f.write("f {}/{}/{} {}/{}/{} {}/{}/{}\n".format(*facestr))
 
 
-def MarchingMesh(array, out_file='model.obj', plot=False, figsize=(4.8,4.8) ):
-
-    '''Marching cubes on sparse 3-D integer `voxelmap` arrays
+def MarchingMesh(
+                array, out_file='model.obj', level=0,
+                spacing=(1., 1., 1.), gradient_direction='descent', step_size=1, 
+                allow_degenerate=True, method='lewiner', mask=None,
+                plot=False, figsize=(4.8,4.8) 
+                ):
+    '''[GLOBAL_METHOD] 
+    Marching cubes on sparse 3-D integer `voxelmap` arrays
 
     Parameters
     ----------
     array: np.array((int/float,int/float,int/float))
-        3-D array for which to run the marching cubes algorithm
+        3-D array for which to run the marching cubes algorithm   
     out_file : str
         name and/or path for Wavefront .obj file output. This is the common format for OpenGL 3-D model files (default: model.obj) 
+
+    --- FROM SKIMAGE.MEASURE.MARCHING_CUBES ---
+    level : float, optional
+        Contour value to search for isosurfaces in `volume`. If not
+        given or None, the average of the min and max of vol is used.
+    spacing : length-3 tuple of floats, optional
+        Voxel spacing in spatial dimensions corresponding to numpy array
+        indexing dimensions (M, N, P) as in `volume`.
+    gradient_direction : string, optional
+        Controls if the mesh was generated from an isosurface with gradient
+        descent toward objects of interest (the default), or the opposite,
+        considering the *left-hand* rule.
+        The two options are:
+        * descent : Object was greater than exterior
+        * ascent : Exterior was greater than object
+
+    step_size : int, optional
+        Step size in voxels. Default 1. Larger steps yield faster but
+        coarser results. The result will always be topologically correct
+        though.
+    allow_degenerate : bool, optional
+        Whether to allow degenerate (i.e. zero-area) triangles in the
+        end-result. Default True. If False, degenerate triangles are
+        removed, at the cost of making the algorithm slower.
+    method: str, optional
+        One of 'lewiner', 'lorensen' or '_lorensen'. Specify which of
+        Lewiner et al. or Lorensen et al. method will be used. The
+        '_lorensen' flag correspond to an old implementation that will
+        be deprecated in version 0.19.
+    mask : (M, N, P) array, optional
+        Boolean array. The marching cube algorithm will be computed only on
+        True elements. This will save computational time when interfaces
+        are located within certain region of the volume M, N, P-e.g. the top
+        half of the cube-and also allow to compute finite surfaces-i.e. open
+        surfaces that do not end at the border of the cube.
+
     plot: bool
         plots a preliminary 3-D triangulated image if True
     '''
 
     '''Adapted from: https://scikit-image.org/docs/stable/auto_examples/edges/plot_marching_cubes.html'''
     # Use marching cubes to obtain the surface mesh of these ellipsoids
-    verts, faces, normals, values = measure.marching_cubes(array, 0)
+    verts, faces, normals, values = measure.marching_cubes(
+                                    array, level=level, spacing=spacing, 
+                                    gradient_direction=gradient_direction, step_size=step_size,
+                                    allow_degenerate=allow_degenerate, method=method, mask=mask
+                                    )
 
     'write wavefront .obj file for generated mesh'
     writeobj_MC(out_file, verts, faces, normals, values)
@@ -201,8 +291,9 @@ def MarchingMesh(array, out_file='model.obj', plot=False, figsize=(4.8,4.8) ):
 
 import voxelmap.objviewer as viewer
 
-def MeshView(objfile, wireframe=False, viewport=(2048, 1152)):
-    '''MeshView [GLOBAL]: triangulated mesh view with OpenGL [ uses pygame ]
+def MeshView(objfile='model.obj', wireframe=False, viewport=(2048, 1152)):
+    '''[GLOBAL_METHOD] 
+    MeshView: triangulated mesh view with OpenGL [ uses pygame ]
 
     Parameters
     ----------
